@@ -24,23 +24,19 @@ import warnings
 import numpy as np
 import pandas as pd
 
+from adcc import dot
+from scipy import constants
+
 from . import adc_pp
 from .misc import cached_property
-from .timings import Timer, timed_member_call
-from .AdcMethod import AdcMethod
+from .timings import timed_member_call
+from .Excitation import Excitation, mark_excitation_property
 from .FormatIndex import (FormatIndexAdcc, FormatIndexBase,
                           FormatIndexHfProvider, FormatIndexHomoLumo)
-from .visualisation import ExcitationSpectrum
 from .OneParticleOperator import product_trace
+from .ElectronicTransition import ElectronicTransition
 from .FormatDominantElements import FormatDominantElements
 
-from adcc import dot
-
-from scipy import constants
-from matplotlib import pyplot as plt
-from .solver.SolverStateBase import EigenSolverStateBase
-from .Excitation import Excitation, mark_excitation_property
-
 
 class FormatExcitationVector:
     def __init__(self, matrix, tolerance=0.01, index_format=None):
@@ -85,8 +81,7 @@ class FormatExcitationVector:
         if not isinstance(vectors, list):
             return self.optimise_formatting([vectors])
         for vector in vectors:
-            for block in self.matrix.blocks:
-                spaces = self.matrix.block_spaces(block)
+            for block, spaces in self.matrix.axis_spaces.items():
                 self.tensor_format.optimise_formatting((spaces, vector[block]))
 
     @property
@@ -95,9 +90,9 @@ class FormatExcitationVector:
         The width of an amplitude line if a tensor is formatted with this class
         """
         # TODO This assumes a PP ADC matrix
-        if self.matrix.blocks == ["s"]:
+        if self.matrix.axis_blocks == ["ph"]:
             nblk = 2
-        elif self.matrix.blocks == ["s", "d"]:
+        elif self.matrix.axis_blocks == ["ph", "pphh"]:
             nblk = 4
         else:
             raise NotImplementedError("Unknown ADC matrix structure")
@@ -109,9 +104,9 @@ class FormatExcitationVector:
     def format(self, vector):
         idxgap = self.index_format.max_n_characters * " "
         # TODO This assumes a PP ADC matrix
-        if self.matrix.blocks == ["s"]:
+        if self.matrix.axis_blocks == ["ph"]:
             formats = {"ov": "{0} -> {1}  {2}->{3}", }
-        elif self.matrix.blocks == ["s", "d"]:
+        elif self.matrix.axis_blocks == ["ph", "pphh"]:
             formats = {
                 "ov":   "{0} " + idxgap + " -> {1} " + idxgap + "  {2} ->{3} ",
                 "oovv": "{0} {1} -> {2} {3}  {4}{5}->{6}{7}",
@@ -120,9 +115,8 @@ class FormatExcitationVector:
             raise NotImplementedError("Unknown ADC matrix structure")
 
         ret = []
-        for block in self.matrix.blocks:
+        for block, spaces in self.matrix.axis_spaces.items():
             # Strip numbers for the lookup into formats above
-            spaces = self.matrix.block_spaces(block)
             stripped = "".join(c for c in "".join(spaces) if c.isalpha())
 
             formatted = self.tensor_format.format_as_list(spaces, vector[block])
@@ -132,11 +126,12 @@ class FormatExcitationVector:
         return "\n".join(ret)
 
 
-class ExcitedStates:
+class ExcitedStates(ElectronicTransition):
     def __init__(self, data, method=None, property_method=None,
                  excitation_energy_corrections={}):
         """Construct an ExcitedStates class from some data obtained
-        from an interative solver.
+        from an interative solver or another :class:`ExcitedStates`
+        object.
 
         The class provides access to the results from an ADC calculation
         as well as derived properties. Properties are computed lazily
@@ -151,7 +146,8 @@ class ExcitedStates:
         ----------
         data
             Any kind of iterative solver state. Typically derived off
-            a :class:`solver.EigenSolverStateBase`.
+            a :class:`solver.EigenSolverStateBase`. Can also be an
+            :class:`ExcitedStates` object.
         method : str, optional
             Provide an explicit method parameter if data contains none.
         property_method : str, optional
@@ -161,68 +157,12 @@ class ExcitedStates:
             Provide a dictionary of functions to compute corrections for
             excitation energies, called for each excitation individually
         """
-        self.matrix = data.matrix
-        self.ground_state = self.matrix.ground_state
-        self.reference_state = self.matrix.ground_state.reference_state
-        self.operators = self.reference_state.operators
+        super().__init__(data, method, property_method)
 
-        # List of all the objects which have timers (do not yet collect
-        # timers, since new times might be added implicitly at a later point)
-        self._property_timer = Timer()
-        self._timed_objects = [("", self.reference_state),
-                               ("adcmatrix", self.matrix),
-                               ("mp", self.ground_state),
-                               ("intermediates", self.matrix.intermediates)]
-        if hasattr(data, "timer"):
-            datakey = getattr(data, "algorithm", data.__class__.__name__)
-            self._timed_objects.append((datakey, data))
-
-        # Copy some optional attributes
-        for optattr in ["converged", "spin_change", "kind", "n_iter"]:
-            if hasattr(data, optattr):
-                setattr(self, optattr, getattr(data, optattr))
-
-        self.method = getattr(data, "method", method)
-        if self.method is None:
-            self.method = self.matrix.method
-        if not isinstance(self.method, AdcMethod):
-            self.method = AdcMethod(self.method)
-        if property_method is None:
-            if self.method.level < 3:
-                property_method = self.method
-            else:
-                # Auto-select ADC(2) properties for ADC(3) calc
-                property_method = self.method.at_level(2)
-        elif not isinstance(property_method, AdcMethod):
-            property_method = AdcMethod(property_method)
-        self.__property_method = property_method
-
-        # Special stuff for special solvers
-        if isinstance(data, EigenSolverStateBase):
-            self.__excitation_vector = data.eigenvectors
-            self.__excitation_energy_uncorrected = data.eigenvalues
-        else:
-            if hasattr(data, "eigenvalues"):
-                self.__excitation_energy_uncorrected = data.eigenvalues
-            if hasattr(data, "eigenvectors"):
-                self.__excitation_vector = data.eigenvectors
-            # if both excitation_energy and excitation_energy_uncorrected
-            # are present, the latter one has priority
-            if hasattr(data, "excitation_energy"):
-                self.__excitation_energy_uncorrected = \
-                    data.excitation_energy.copy()
-            if hasattr(data, "excitation_energy_uncorrected"):
-                self.__excitation_energy_uncorrected =\
-                    data.excitation_energy_uncorrected.copy()
-            if hasattr(data, "excitation_vector"):
-                self.__excitation_vector = data.excitation_vector
-            if hasattr(data, "excitation_energy_corrections"):
-                merged = data.excitation_energy_corrections.copy()
-                merged.update(excitation_energy_corrections)
-                excitation_energy_corrections = merged
-
-        # Collect all excitation energy corrections
-        self.__excitation_energy = self.__excitation_energy_uncorrected.copy()
+        if hasattr(data, "excitation_energy_corrections"):
+            merged = data.excitation_energy_corrections.copy()
+            merged.update(excitation_energy_corrections)
+            excitation_energy_corrections = merged
 
         if hasattr(self.reference_state, "excitation_energy_corrections"):
             merged = self.reference_state.excitation_energy_corrections.copy()
@@ -239,47 +179,7 @@ class ExcitedStates:
             correction = np.array([corr_function(exci)
                                    for exci in self.excitations])
             setattr(self, key, correction)
-            self.__excitation_energy += correction
-
-    def __len__(self):
-        return self.size
-
-    @property
-    def size(self):
-        return self.__excitation_energy.size
-
-    @property
-    def timer(self):
-        """Return a cumulative timer collecting timings from the calculation"""
-        ret = Timer()
-        for key, obj in self._timed_objects:
-            ret.attach(obj.timer, subtree=key)
-        ret.attach(self._property_timer, subtree="properties")
-        ret.time_construction = self.reference_state.timer.time_construction
-        return ret
-
-    @property
-    def property_method(self):
-        """The method used to evaluate ADC properties"""
-        return self.__property_method
-
-    @property
-    @mark_excitation_property()
-    def excitation_energy(self):
-        """Excitation energies including all corrections in atomic units"""
-        return self.__excitation_energy
-
-    @property
-    @mark_excitation_property()
-    def excitation_energy_uncorrected(self):
-        """Excitation energies without any corrections in atomic units"""
-        return self.__excitation_energy_uncorrected
-
-    @property
-    @mark_excitation_property()
-    def excitation_vector(self):
-        """List of excitation vectors"""
-        return self.__excitation_vector
+            self._excitation_energy += correction
 
     @cached_property
     @mark_excitation_property(transform_to_ao=True)
@@ -290,81 +190,6 @@ class ExcitedStates:
                                      evec, self.matrix.intermediates)
                 for evec in self.excitation_vector]
 
-    @cached_property
-    @mark_excitation_property()
-    @timed_member_call(timer="_property_timer")
-    def transition_dipole_moment(self):
-        """List of transition dipole moments of all computed states"""
-        if self.property_method.level == 0:
-            warnings.warn("ADC(0) transition dipole moments are known to be "
-                          "faulty in some cases.")
-        dipole_integrals = self.operators.electric_dipole
-        return np.array([
-            [product_trace(comp, tdm) for comp in dipole_integrals]
-            for tdm in self.transition_dm
-        ])
-
-    @cached_property
-    @mark_excitation_property()
-    @timed_member_call(timer="_property_timer")
-    def transition_dipole_moment_velocity(self):
-        """List of transition dipole moments in the
-        velocity gauge of all computed states"""
-        if self.property_method.level == 0:
-            warnings.warn("ADC(0) transition velocity dipole moments "
-                          "are known to be faulty in some cases.")
-        dipole_integrals = self.operators.nabla
-        return np.array([
-            [product_trace(comp, tdm) for comp in dipole_integrals]
-            for tdm in self.transition_dm
-        ])
-
-    @cached_property
-    @mark_excitation_property()
-    @timed_member_call(timer="_property_timer")
-    def transition_magnetic_dipole_moment(self):
-        """List of transition magnetic dipole moments of all computed states"""
-        if self.property_method.level == 0:
-            warnings.warn("ADC(0) transition magnetic dipole moments "
-                          "are known to be faulty in some cases.")
-        mag_dipole_integrals = self.operators.magnetic_dipole
-        return np.array([
-            [product_trace(comp, tdm) for comp in mag_dipole_integrals]
-            for tdm in self.transition_dm
-        ])
-
-    @cached_property
-    @mark_excitation_property()
-    def oscillator_strength(self):
-        """List of oscillator strengths of all computed states"""
-        return 2. / 3. * np.array([
-            np.linalg.norm(tdm)**2 * np.abs(ev)
-            for tdm, ev in zip(self.transition_dipole_moment,
-                               self.excitation_energy)
-        ])
-
-    @cached_property
-    @mark_excitation_property()
-    def oscillator_strength_velocity(self):
-        """List of oscillator strengths in
-        velocity gauge of all computed states"""
-        return 2. / 3. * np.array([
-            np.linalg.norm(tdm)**2 / np.abs(ev)
-            for tdm, ev in zip(self.transition_dipole_moment_velocity,
-                               self.excitation_energy)
-        ])
-
-    @cached_property
-    @mark_excitation_property()
-    def rotatory_strength(self):
-        """List of rotatory strengths of all computed states"""
-        return np.array([
-            np.dot(tdm, magmom) / ee
-            for tdm, magmom, ee in zip(self.transition_dipole_moment_velocity,
-                                       self.transition_magnetic_dipole_moment,
-                                       self.excitation_energy)
-        ])
-
     @cached_property
     @mark_excitation_property(transform_to_ao=True)
     @timed_member_call(timer="_property_timer")
@@ -398,103 +223,6 @@ class ExcitedStates:
             for ddm in self.state_diffdm
         ])
 
-    @property
-    @mark_excitation_property()
-    def cross_section(self):
-        """List of one-photon absorption cross sections of all computed states"""
-        # TODO Source?
-        fine_structure = constants.fine_structure
-        fine_structure_au = 1 / fine_structure
-        prefac = 2.0 * np.pi ** 2 / fine_structure_au
-        return prefac * self.oscillator_strength
-
-    def plot_spectrum(self, broadening="lorentzian", xaxis="eV",
-                      yaxis="cross_section", width=0.01, **kwargs):
-        """One-shot plotting function for the spectrum generated by all states
-        known to this class.
-
-        Makes use of the :class:`adcc.visualisation.ExcitationSpectrum` class
-        in order to generate and format the spectrum to be plotted, using
-        many sensible defaults.
-
-        Parameters
-        ----------
-        broadening : str or None or callable, optional
-            The broadening type to used for the computed excitations.
-            A value of None disables broadening any other value is passed
-            straight to
-            :func:`adcc.visualisation.ExcitationSpectrum.broaden_lines`.
-        xaxis : str
-            Energy unit to be used on the x-Axis. Options:
-            ["eV", "au", "nm", "cm-1"]
-        yaxis : str
-            Quantity to plot on the y-Axis. Options are "cross_section",
-            "osc_strength", "dipole" (plots norm of transition dipole),
-            "rotational_strength" (ECD spectrum with rotational strength)
-        width : float, optional
-            Gaussian broadening standard deviation or Lorentzian broadening
-            gamma parameter. The value should be given in atomic units
-            and will be converted to the unit of the energy axis.
-        """
-        if xaxis == "eV":
-            eV = constants.value("Hartree energy in eV")
-            energies = self.excitation_energy * eV
-            width = width * eV
-            xlabel = "Energy (eV)"
-        elif xaxis in ["au", "Hartree", "a.u."]:
-            energies = self.excitation_energy
-            xlabel = "Energy (au)"
-        elif xaxis == "nm":
-            hc = constants.h * constants.c
-            Eh = constants.value("Hartree energy")
-            energies = hc / (self.excitation_energy * Eh) * 1e9
-            xlabel = "Wavelength (nm)"
-            if broadening is not None and not callable(broadening):
-                raise ValueError("xaxis=nm and broadening enabled is "
-                                 "not supported.")
-        elif xaxis in ["cm-1", "cm^-1", "cm^{-1}"]:
-            towvn = constants.value("hartree-inverse meter relationship") / 100
-            energies = self.excitation_energy * towvn
-            width = width * towvn
-            xlabel = "Wavenumbers (cm^{-1})"
-        else:
-            raise ValueError("Unknown xaxis specifier: {}".format(xaxis))
-
-        if yaxis in ["osc", "osc_strength", "oscillator_strength", "f"]:
-            absorption = self.oscillator_strength
-            ylabel = "Oscillator strengths (au)"
-        elif yaxis in ["dipole", "dipole_norm", "μ"]:
-            absorption = np.linalg.norm(self.transition_dipole_moment, axis=1)
-            ylabel = "Modulus of transition dipole (au)"
-        elif yaxis in ["cross_section", "σ"]:
-            absorption = self.cross_section
-            ylabel = "Cross section (au)"
-        elif yaxis in ["rot", "rotational_strength", "rotatory_strength"]:
-            absorption = self.rotatory_strength
-            ylabel = "Rotatory strength (au)"
-        else:
-            raise ValueError("Unknown yaxis specifier: {}".format(yaxis))
-
-        sp = ExcitationSpectrum(energies, absorption)
-        sp.xlabel = xlabel
-        sp.ylabel = ylabel
-        if not broadening:
-            plots = sp.plot(style="discrete", **kwargs)
-        else:
-            kwdisc = kwargs.copy()
-            kwdisc.pop("label", "")
-            plots = sp.plot(style="discrete", **kwdisc)
-
-            kwargs.pop("color", "")
-            sp_broad = sp.broaden_lines(width, shape=broadening)
-            plots.extend(sp_broad.plot(color=plots[0].get_color(),
-                                       style="continuous", **kwargs))
-
-        if xaxis in ["nm"]:
-            # Invert x axis
-            plt.xlim(plt.xlim()[::-1])
-        return plots
-
     def describe(self, oscillator_strengths=True, rotatory_strengths=False,
                  state_dipole_moments=False, transition_dipole_moments=False,
                  block_norms=True):
@@ -520,7 +248,7 @@ class ExcitedStates:
             by default ``True``.
         """
         # TODO This function is quite horrible and definitely needs some
-        #      refactoring
+        #      refactoring, also it assumes ADC-PP everywhere
 
         eV = constants.value("Hartree energy in eV")
         has_dipole = "electric_dipole" in self.operators.available
@@ -545,14 +273,14 @@ class ExcitedStates:
             opt_body += "{rot:8.4f} "
             opt_thead += " rot str "
             opt["rot"] = lambda i, vec: self.rotatory_strength[i]
-        if "s" in self.matrix.blocks and block_norms:
+        if "ph" in self.matrix.axis_blocks and block_norms:
             opt_body += "{v1:9.4g} "
             opt_thead += "   |v1|^2 "
-            opt["v1"] = lambda i, vec: dot(vec["s"], vec["s"])
-        if "d" in self.matrix.blocks and block_norms:
+            opt["v1"] = lambda i, vec: dot(vec.ph, vec.ph)
+        if "pphh" in self.matrix.axis_blocks and block_norms:
             opt_body += "{v2:9.4g} "
             opt_thead += "   |v2|^2 "
-            opt["v2"] = lambda i, vec: dot(vec["d"], vec["d"])
+            opt["v2"] = lambda i, vec: dot(vec.pphh, vec.pphh)
         if has_dipole and state_dipole_moments:
             opt_body += " {dmx:8.4f} {dmy:8.4f} {dmz:8.4f}"
             opt_thead += "     state dipole moment   "
@@ -719,6 +447,39 @@ class ExcitedStates:
                 ret.append(key)
         return ret
 
+    def to_qcvars(self, properties=False, recurse=False):
+        """
+        Return a dictionary with property keys compatible to a Psi4 wavefunction
+        or a QCEngine Atomicresults object.
+        """
+        name = self.method.name.upper()
+
+        qcvars = {
+            "EXCITATION KIND": self.kind.upper(),
+            "NUMBER OF EXCITED STATES": len(self.excitation_energy),
+            f"{name} ITERATIONS": self.n_iter,
+            f"{name} EXCITATION ENERGIES": self.excitation_energy,
+        }
+
+        if properties:
+            qcvars.update({
+                # Transition properties
+                f"{name} TRANSITION DIPOLES (LEN)": self.transition_dipole_moment,
+                f"{name} TRANSITION DIPOLES (VEL)": self.transition_dipole_moment_velocity,  # noqa: E501
+                f"{name} OSCILLATOR STRENGTHS (LEN)": self.oscillator_strength,
+                f"{name} OSCILLATOR STRENGTHS (VEL)": self.oscillator_strength_velocity,  # noqa: E501
+                f"{name} ROTATIONAL STRENGTHS (VEL)": self.rotatory_strength,
+                #
+                # State properties
+                f"{name} STATE DIPOLES": self.state_dipole_moment
+            })
+
+        if recurse:
+            mpvars = self.ground_state.to_qcvars(properties, recurse=True,
+                                                 maxlevel=self.method.level)
+            qcvars.update(mpvars)
+        return qcvars
+
     @property
     def excitations(self):
         """
