@@ -20,9 +20,10 @@
 ## along with adcc. If not, see <http://www.gnu.org/licenses/>.
 ##
 ## ---------------------------------------------------------------------
+from adcc.workflow import InputError
 
 
-class InvalidReference(ValueError):
+class InvalidReference(InputError):
     """
     Exception thrown if a passed SCF reference is invalid, e.g. because
     a feature like density-fitting has been applied, which is inconsistent
